var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { loadPageChunk, queryCollection } from './client';
import * as _ from 'lodash';
function mergeRecordMap(maps) {
    const result = {};
    for (const map of maps) {
        if (map === undefined) {
            continue;
        }
        for (const key of Object.keys(map)) {
            if (result.hasOwnProperty(key)) {
                continue;
            }
            result[key] = map[key];
        }
    }
    if (Object.keys(result).length > 0) {
        return result;
    }
}
/**
 * example: https://www.notion.so/notinktest/Get-Started-1d748958865c4cf397d6c996756cd77e
 * @param blockId: 1d748958865c4cf397d6c996756cd77e
 * @return real block id: 1d748958-865c-4cf3-97d6-c996756cd77e
 */
export function getFullBlockId(blockId) {
    if (blockId.match('^[a-zA-Z0-9]+$')) {
        return [
            blockId.substr(0, 8),
            blockId.substr(8, 4),
            blockId.substr(12, 4),
            blockId.substr(16, 4),
            blockId.substr(20, 32),
        ].join('-');
    }
    return blockId;
}
export function loadFullPageChunk(pageId) {
    return __awaiter(this, void 0, void 0, function* () {
        let cursor = { stack: [] };
        let chunkNumber = 0;
        const limit = 50;
        const chunks = [];
        do {
            const pageChunk = yield loadPageChunk({
                chunkNumber,
                limit,
                pageId,
                verticalColumns: true,
                cursor,
            });
            cursor = pageChunk.cursor;
            chunkNumber++;
            chunks.push(pageChunk);
        } while (cursor && cursor.stack.length > 0);
        return {
            cursor: { stack: [] },
            recordMap: {
                block: mergeRecordMap(chunks.map(it => it.recordMap.block)),
                collection: mergeRecordMap(chunks.map(it => it.recordMap.collection)),
                collection_view: mergeRecordMap(chunks.map(it => it.recordMap.collection_view)),
                notion_user: mergeRecordMap(chunks.map(it => it.recordMap.notion_user)),
                space: mergeRecordMap(chunks.map(it => it.recordMap.space)),
            },
        };
    });
}
export function loadBlockTree(pageId) {
    return __awaiter(this, void 0, void 0, function* () {
        const pageChunk = yield loadFullPageChunk(pageId);
        const getBlockNode = (blockId) => {
            if (!pageChunk.recordMap.block) {
                return;
            }
            const value = pageChunk.recordMap.block[blockId].value;
            const content = value.content;
            const children = content ? _.compact(content.map(it => getBlockNode(it))) : undefined;
            return {
                value,
                children,
            };
        };
        return getBlockNode(pageId);
    });
}
export function loadCollectionItems(data) {
    return __awaiter(this, void 0, void 0, function* () {
        let queryResult = yield queryCollection(data);
        if (queryResult.result.total > data.loader.limit) {
            data.loader.limit = Math.ceil(queryResult.result.total / 70) * 70;
            queryResult = yield queryCollection(data);
        }
        if (queryResult.recordMap.collection === undefined ||
            queryResult.recordMap.collection_view === undefined) {
            return [];
        }
        const collection = queryResult.recordMap.collection[data.collectionId];
        const collectionView = queryResult.recordMap.collection_view[data.collectionViewId];
        const block = queryResult.recordMap.block;
        if (block === undefined) {
            return [];
        }
        const schema = collection.value.schema;
        return collectionView.value.page_sort
            .map(it => block[it])
            .filter(it => !!it)
            .map(it => {
            const properties = it.value.properties ? it.value.properties : {};
            const result = {
                page: it,
                properties: {},
            };
            for (const k of Object.keys(schema)) {
                result.properties[k] = {
                    value: properties[schema[k].name],
                    schema: schema[k],
                };
            }
            return result;
        });
    });
}
//# sourceMappingURL=helper.js.map